'''
Copyright (c) 2014 Pivotal Software, Inc.  All Rights Reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0.
Unless required by applicable law or agreed to in writing, software distributed under the License
is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and limitations under the License.
'''

from distutils.core import setup

setup(
    name='gemfire-rest',
    version='1.0.1',
    description='Python client for GemFire\'s REST service ',
    long_description=open('README.txt').read(),
    url='https://github.com/gemfire/py-gemfire-rest',
    packages=['gemfire-client',],
    license='Apache 2.0',
    keywords=['GemFire', 'key-value store'],
    install_requires=['logging', 'requests', 'jsonpickle'],
    author='Madhura Kulkarni, Anirudh Jain',
    author_email='madhurak25@gmail.com',
    maintainer='Swapnil Bawaskar',
    maintainer_email='sbawaskar@pivotal.io',
)
